@@ -1,7 +1,7 @@
 //========================================================================
 // A simple particle engine with threaded physics
 // Copyright (c) Marcus Geelnard
-// Copyright (c) Camilla Berglund <elmindreda@elmindreda.org>
+// Copyright (c) Camilla Löwy <elmindreda@glfw.org>
 //
 // This software is provided 'as-is', without any express or implied
 // warranty. In no event will the authors be held liable for any damages
@@ -39,6 +39,8 @@
 #include <getopt.h>
 #include <linmath.h>
 
+#include <glad/gl.h>
+#define GLFW_INCLUDE_NONE
 #include <GLFW/glfw3.h>
 
 // Define tokens for GL_EXT_separate_specular_color if not already defined
@@ -442,7 +444,7 @@ static void draw_particles(GLFWwindow* window, double t, float dt)
     }
 
     // Set up vertex arrays. We use interleaved arrays, which is easier to
-    // handle (in most situations) and it gives a linear memeory access
+    // handle (in most situations) and it gives a linear memory access
     // access pattern (which may give better performance in some
     // situations). GL_T2F_C4UB_V3F means: 2 floats for texture coords,
     // 4 ubytes for color and 3 floats for vertex coord (in that order).
@@ -456,7 +458,9 @@ static void draw_particles(GLFWwindow* window, double t, float dt)
     {
         struct timespec ts;
         clock_gettime(CLOCK_REALTIME, &ts);
-        ts.tv_nsec += 100000000;
+        ts.tv_nsec += 100 * 1000 * 1000;
+        ts.tv_sec += ts.tv_nsec / (1000 * 1000 * 1000);
+        ts.tv_nsec %= 1000 * 1000 * 1000;
         cnd_timedwait(&thread_sync.p_done, &thread_sync.particles_lock, &ts);
     }
 
@@ -650,7 +654,7 @@ static void draw_fountain(void)
 
 
 //========================================================================
-// Recursive function for building variable tesselated floor
+// Recursive function for building variable tessellated floor
 //========================================================================
 
 static void tessellate_floor(float x1, float y1, float x2, float y2, int depth)
@@ -717,7 +721,7 @@ static void draw_floor(void)
         glMaterialfv(GL_FRONT, GL_SPECULAR, floor_specular);
         glMaterialf(GL_FRONT, GL_SHININESS, floor_shininess);
 
-        // Draw floor as a bunch of triangle strips (high tesselation
+        // Draw floor as a bunch of triangle strips (high tessellation
         // improves lighting)
         glNormal3f(0.f, 0.f, 1.f);
         glBegin(GL_QUADS);
@@ -875,7 +879,7 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
         switch (key)
         {
             case GLFW_KEY_ESCAPE:
-                glfwSetWindowShouldClose(window, GL_TRUE);
+                glfwSetWindowShouldClose(window, GLFW_TRUE);
                 break;
             case GLFW_KEY_W:
                 wireframe = !wireframe;
@@ -907,7 +911,9 @@ static int physics_thread_main(void* arg)
         {
             struct timespec ts;
             clock_gettime(CLOCK_REALTIME, &ts);
-            ts.tv_nsec += 100000000;
+            ts.tv_nsec += 100 * 1000 * 1000;
+            ts.tv_sec += ts.tv_nsec / (1000 * 1000 * 1000);
+            ts.tv_nsec %= 1000 * 1000 * 1000;
             cnd_timedwait(&thread_sync.d_done, &thread_sync.particles_lock, &ts);
         }
 
@@ -989,6 +995,7 @@ int main(int argc, char** argv)
         glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
 
     glfwMakeContextCurrent(window);
+    gladLoadGL(glfwGetProcAddress);
     glfwSwapInterval(1);
 
     glfwSetFramebufferSizeCallback(window, resize_callback);
